@@ -53,6 +53,9 @@ PORT_SCHEMA = {
         'pxe_enabled': {'type': ['string', 'boolean', 'null']},
         'uuid': {'type': ['string', 'null']},
         'name': {'type': ['string', 'null']},
+        'description': {'type': ['string', 'null'], 'maxLength': 255},
+        'vendor': {'type': ['string', 'null'], 'maxLength': 32},
+        'category': {'type': ['string', 'null'], 'maxLength': 80},
     },
     'required': ['address'],
     'oneOf': [
@@ -76,6 +79,9 @@ PATCH_ALLOWED_FIELDS = [
     'portgroup_uuid',
     'pxe_enabled',
     'name',
+    'description',
+    'vendor',
+    'category',
 ]
 
 PORT_VALIDATOR_EXTRA = args.dict_valid(
@@ -134,6 +140,15 @@ def hide_fields_in_newer_versions(port):
             # expect the key port.local_link_connection to exist even if we
             # cannot set a valid value
             port['local_link_connection'] = {}
+    # if requested version is < 1.97, hide description field.
+    if not api_utils.allow_port_description():
+        port.pop('description', None)
+    # if requested version is < 1.100, hide vendor field.
+    if not api_utils.allow_port_vendor():
+        port.pop('vendor', None)
+    # if requested version is < 1.101, hide category field.
+    if not api_utils.allow_port_category():
+        port.pop('category', None)
 
 
 def convert_with_links(rpc_port, fields=None, sanitize=True):
@@ -150,6 +165,7 @@ def convert_with_links(rpc_port, fields=None, sanitize=True):
             'pxe_enabled',
             'node_uuid',
             'name',
+            'description',
         )
     )
     if rpc_port.portgroup_id:
@@ -227,7 +243,8 @@ class PortsController(rest.RestController):
     def _get_ports_collection(self, node_ident, address, portgroup_ident,
                               shard, marker, limit, sort_key, sort_dir,
                               resource_url=None, fields=None, detail=None,
-                              project=None):
+                              project=None, description_contains=None,
+                              conductor_groups=None):
         """Retrieve a collection of ports.
 
         :param node_ident: UUID or name of a node, to get only ports for that
@@ -250,6 +267,10 @@ class PortsController(rest.RestController):
             of the resource to be returned.
         :param detail: Optional, show detailed list of ports
         :param project: Optional, filter by project
+        :param description_contains: Optional string value to get only ports
+                                     with description field contains matching
+                                     value.
+        :param conductor_groups: conductor groups, to filter the request by.
         :returns: a list of ports.
 
         """
@@ -277,6 +298,15 @@ class PortsController(rest.RestController):
             if exclusive_filters > 1:
                 raise exception.OperationNotPermitted()
 
+        filters = {}
+        if description_contains:
+            filters['description_contains'] = description_contains
+
+        if conductor_groups and (node_ident or portgroup_ident or address):
+            raise exception.Invalid(
+                _("Filtering by conductor_groups cannot be combined with "
+                  "node_ident, portgroup_ident, or node address filters."))
+
         if portgroup_ident:
             # FIXME: Since all we need is the portgroup ID, we can
             #                 make this more efficient by only querying
@@ -288,7 +318,8 @@ class PortsController(rest.RestController):
                                                       marker_obj,
                                                       sort_key=sort_key,
                                                       sort_dir=sort_dir,
-                                                      project=project)
+                                                      project=project,
+                                                      filters=filters)
         elif node_ident:
             # FIXME(comstud): Since all we need is the node ID, we can
             #                 make this more efficient by only querying
@@ -299,18 +330,22 @@ class PortsController(rest.RestController):
                                                  node.id, limit, marker_obj,
                                                  sort_key=sort_key,
                                                  sort_dir=sort_dir,
-                                                 project=project)
+                                                 project=project,
+                                                 filters=filters)
         elif address:
             ports = self._get_ports_by_address(address, project=project)
         elif shard:
             ports = objects.Port.list_by_node_shards(api.request.context,
                                                      shard, limit,
                                                      marker_obj, sort_key,
-                                                     sort_dir, project=project)
+                                                     sort_dir, project=project,
+                                                     filters=filters)
         else:
             ports = objects.Port.list(api.request.context, limit,
                                       marker_obj, sort_key=sort_key,
-                                      sort_dir=sort_dir, project=project)
+                                      sort_dir=sort_dir, project=project,
+                                      conductor_groups=conductor_groups,
+                                      filters=filters)
         parameters = {}
 
         if detail is not None:
@@ -377,6 +412,15 @@ class PortsController(rest.RestController):
         if ('name' in fields
                 and not api_utils.allow_port_name()):
             raise exception.NotAcceptable()
+        if ('description' in fields
+                and not api_utils.allow_port_description()):
+            raise exception.NotAcceptable()
+        if ('vendor' in fields
+                and not api_utils.allow_port_vendor()):
+            raise exception.NotAcceptable()
+        if ('category' in fields
+                and not api_utils.allow_port_category()):
+            raise exception.NotAcceptable()
 
     @METRICS.timer('PortsController.get_all')
     @method.expose()
@@ -385,10 +429,12 @@ class PortsController(rest.RestController):
                    limit=args.integer, sort_key=args.string,
                    sort_dir=args.string, fields=args.string_list,
                    portgroup=args.uuid_or_name, detail=args.boolean,
-                   shard=args.string_list)
+                   shard=args.string_list, description_contains=args.string,
+                   conductor_groups=args.string_list)
     def get_all(self, node=None, node_uuid=None, address=None, marker=None,
                 limit=None, sort_key='id', sort_dir='asc', fields=None,
-                portgroup=None, detail=None, shard=None):
+                portgroup=None, detail=None, shard=None,
+                description_contains=None, conductor_groups=None):
         """Retrieve a list of ports.
 
         Note that the 'node_uuid' interface is deprecated in favour
@@ -413,6 +459,10 @@ class PortsController(rest.RestController):
                                    for that portgroup.
         :param shard: Optional, a list of shard ids to filter by, only ports
                       associated with nodes in these shards will be returned.
+        :param description_contains: Optional string value to get only ports
+                                     with description field contains matching
+                                     value.
+        :param conductor_groups: conductor groups, to filter the request by.
         :raises: NotAcceptable, HTTPNotFound
         """
         project = api_utils.check_port_list_policy(
@@ -437,6 +487,8 @@ class PortsController(rest.RestController):
         fields = api_utils.get_request_return_fields(fields, detail,
                                                      _DEFAULT_RETURN_FIELDS)
 
+        extra_args = {'description_contains': description_contains}
+
         if not node_uuid and node:
             # We're invoking this interface using positional notation, or
             # explicitly using 'node'.  Try and determine which one.
@@ -450,7 +502,9 @@ class PortsController(rest.RestController):
                                           sort_key, sort_dir,
                                           resource_url='ports',
                                           fields=fields, detail=detail,
-                                          project=project)
+                                          project=project,
+                                          conductor_groups=conductor_groups,
+                                          **extra_args)
 
     @METRICS.timer('PortsController.detail')
     @method.expose()
@@ -458,10 +512,11 @@ class PortsController(rest.RestController):
                    address=args.mac_address, marker=args.uuid,
                    limit=args.integer, sort_key=args.string,
                    sort_dir=args.string, portgroup=args.uuid_or_name,
-                   shard=args.string_list)
+                   shard=args.string_list, description_contains=args.string,
+                   conductor_groups=args.string_list)
     def detail(self, node=None, node_uuid=None, address=None, marker=None,
                limit=None, sort_key='id', sort_dir='asc', portgroup=None,
-               shard=None):
+               shard=None, description_contains=None, conductor_groups=None):
         """Retrieve a list of ports with detail.
 
         Note that the 'node_uuid' interface is deprecated in favour
@@ -484,6 +539,10 @@ class PortsController(rest.RestController):
                       max_limit resources will be returned.
         :param sort_key: column to sort results by. Default: id.
         :param sort_dir: direction to sort. "asc" or "desc". Default: asc.
+        :param description_contains: Optional string value to get only ports
+                                    with description field contains matching
+                                    value.
+        :param conductor_groups: conductor groups, to filter the request by.
         :raises: NotAcceptable, HTTPNotFound
         """
         project = api_utils.check_port_list_policy(
@@ -509,11 +568,14 @@ class PortsController(rest.RestController):
         if parent != "ports":
             raise exception.HTTPNotFound()
 
+        extra_args = {'description_contains': description_contains}
         return self._get_ports_collection(node_uuid or node, address,
                                           portgroup, shard, marker, limit,
                                           sort_key, sort_dir,
                                           resource_url='ports/detail',
-                                          project=project)
+                                          project=project,
+                                          conductor_groups=conductor_groups,
+                                          **extra_args)
 
     @METRICS.timer('PortsController.get_one')
     @method.expose()
@@ -527,7 +589,7 @@ class PortsController(rest.RestController):
         :raises: NotAcceptable, HTTPNotFound
         """
         if self.parent_node_ident or self.parent_portgroup_ident:
-            raise exception.OperationNotPermitted()
+            raise exception.MalformedRequestURI()
 
         rpc_port, rpc_node = api_utils.check_port_policy_and_retrieve(
             'baremetal:port:get', port_ident)
@@ -548,7 +610,7 @@ class PortsController(rest.RestController):
         :raises: NotAcceptable, HTTPNotFound, Conflict
         """
         if self.parent_node_ident or self.parent_portgroup_ident:
-            raise exception.OperationNotPermitted()
+            raise exception.MalformedRequestURI()
 
         # NOTE(lucasagomes): Create the node_id attribute on-the-fly
         #                    to satisfy the api -> rpc object
@@ -670,7 +732,7 @@ class PortsController(rest.RestController):
         :raises: NotAcceptable, HTTPNotFound
         """
         if self.parent_node_ident or self.parent_portgroup_ident:
-            raise exception.OperationNotPermitted()
+            raise exception.MalformedRequestURI()
 
         api_utils.patch_validate_allowed_fields(patch, PATCH_ALLOWED_FIELDS)
 
@@ -754,8 +816,15 @@ class PortsController(rest.RestController):
         if (api_utils.is_path_updated(patch, '/physical_network')
             and rpc_port['physical_network'] is not None
                 and not rpc_port['physical_network']):
-            raise exception.Invalid('A non-empty value is required when '
-                                    'setting physical_network')
+            raise exception.Invalid(_('A non-empty value is required when '
+                                      'setting physical_network'))
+
+        if (api_utils.allow_portgroup_physical_network()
+            and api_utils.is_path_updated(patch, '/physical_network')
+            and portgroup is not None):
+            raise exception.Invalid(
+                _('physical_network cannot be changed while the port belongs '
+                  'to a portgroup.'))
 
         notify_extra = {'node_uuid': rpc_node.uuid,
                         'portgroup_uuid': portgroup and portgroup.uuid or None}
@@ -780,10 +849,10 @@ class PortsController(rest.RestController):
         """Delete a port.
 
         :param port_uuid: UUID of a port.
-        :raises: OperationNotPermitted, HTTPNotFound
+        :raises: MalformedRequestURI, HTTPNotFound
         """
         if self.parent_node_ident or self.parent_portgroup_ident:
-            raise exception.OperationNotPermitted()
+            raise exception.MalformedRequestURI()
 
         rpc_port, rpc_node = api_utils.check_port_policy_and_retrieve(
             'baremetal:port:delete', port_uuid)
