@@ -452,6 +452,8 @@ def sanitize_dict(to_sanitize, fields):
             to_sanitize.pop(key, None)
 
 
+# TODO(stephenfin): We can repurpose this check to only set a default for limit
+# when all APIs use jsonschema validation
 def validate_limit(limit):
     if limit is None:
         return CONF.api.max_limit
@@ -462,6 +464,8 @@ def validate_limit(limit):
     return min(CONF.api.max_limit, limit)
 
 
+# TODO(stephenfin): We can remove this check when all APIs use jsonschema
+# validation
 def validate_sort_dir(sort_dir):
     if sort_dir not in ['asc', 'desc']:
         raise exception.ClientSideError(_("Invalid sort direction: %s. "
@@ -893,6 +897,7 @@ VERSIONED_FIELDS = {
     'firmware_interface': versions.MINOR_86_FIRMWARE_INTERFACE,
     'service_step': versions.MINOR_87_SERVICE,
     'disable_power_off': versions.MINOR_95_DISABLE_POWER_OFF,
+    'instance_name': versions.MINOR_104_NODE_INSTANCE_NAME,
 }
 
 for field in V31_FIELDS:
@@ -1289,6 +1294,26 @@ def allow_portgroup_mode_properties():
             >= versions.MINOR_26_PORTGROUP_MODE_PROPERTIES)
 
 
+def allow_portgroup_physical_network():
+    """Check if physical_network can be added to/queried from a portgroup.
+
+    Version 1.102 of the API added physical_network field to portgroup
+    object.
+    """
+    return (api.request.version.minor
+            >= versions.MINOR_102_PORTGROUP_PHYSICAL_NETWORK)
+
+
+def allow_portgroup_category():
+    """Check if category can be added to/queried from a portgroup.
+
+    Version 1.103 of the API added category field to portgroup
+    object.
+    """
+    return (api.request.version.minor
+            >= versions.MINOR_103_PORTGROUP_CATEGORY)
+
+
 def allow_vifs_subcontroller():
     """Check if node/vifs can be used.
 
@@ -2259,3 +2284,17 @@ def allow_port_category():
     Version 1.101 of the API added category field to the port object.
     """
     return api.request.version.minor >= versions.MINOR_101_PORT_CATEGORY
+
+
+def allow_node_instance_name():
+    """Check if instance_name is allowed for nodes.
+
+    Version 1.104 of the API added instance_name field to the node object.
+    """
+    return api.request.version.minor >= versions.MINOR_104_NODE_INSTANCE_NAME
+
+
+def check_allow_filter_by_instance_name(instance_name):
+    if instance_name is not None and not allow_node_instance_name():
+        raise exception.NotAcceptable(
+            _("instance_name is not acceptable in this API version"))
