@@ -23,6 +23,7 @@ _opts = [
     ('ansible', ironic.conf.ansible.opts),
     ('api', ironic.conf.api.opts),
     ('audit', ironic.conf.audit.opts),
+    ('auto_discovery', ironic.conf.inspector.discovery_opts),
     ('cinder', ironic.conf.cinder.list_opts()),
     ('conductor', ironic.conf.conductor.opts),
     ('console', ironic.conf.console.opts),
@@ -35,12 +36,12 @@ _opts = [
     ('glance', ironic.conf.glance.list_opts()),
     ('healthcheck', ironic.conf.healthcheck.opts),
     ('ilo', ironic.conf.ilo.opts),
-    ('inspector', ironic.conf.inspector.list_opts()),
+    ('inspection_rules', ironic.conf.inspector.inspection_rule_opts),
+    ('inspector', ironic.conf.inspector.opts),
     ('inventory', ironic.conf.inventory.opts),
     ('ipmi', ironic.conf.ipmi.opts),
     ('irmc', ironic.conf.irmc.opts),
     ('json_rpc', ironic.conf.json_rpc.list_opts()),
-    ('local_rpc', ironic.conf.local_rpc.list_opts()),
     ('mdns', ironic.conf.mdns.opts),
     ('metrics', ironic.conf.metrics.opts),
     ('metrics_statsd', ironic.conf.metrics.statsd_opts),
@@ -49,6 +50,7 @@ _opts = [
     ('nova', ironic.conf.nova.list_opts()),
     ('oci', ironic.conf.oci.opts),
     ('pxe', ironic.conf.pxe.opts),
+    ('pxe_filter', ironic.conf.inspector.pxe_filter_opts),
     ('redfish', ironic.conf.redfish.opts),
     ('sensor_data', ironic.conf.sensor_data.opts),
     ('service_catalog', ironic.conf.service_catalog.list_opts()),
@@ -86,7 +88,6 @@ def update_opt_defaults():
             'oslo.messaging=INFO',
             'oslo_messaging=INFO',
             'stevedore=INFO',
-            'eventlet.wsgi.server=INFO',
             'iso8601=WARNING',
             'requests=WARNING',
             'urllib3.connectionpool=WARNING',
