@@ -36,6 +36,7 @@ from ironic.conf import CONF
 from ironic.drivers.modules import boot_mode_utils
 from ironic.drivers.modules import deploy_utils
 from ironic.drivers.modules import image_cache
+from ironic.drivers.modules import inspect_utils
 from ironic.drivers import utils as driver_utils
 
 LOG = log.getLogger(__name__)
@@ -186,8 +187,19 @@ def override_api_url(params):
         return params
 
     params = params or {}
-    params[deploy_utils.IPA_URL_PARAM_NAME] = \
-        CONF.deploy.external_callback_url.rstrip('/')
+    override_callback_url = CONF.deploy.external_callback_url.rstrip('/')
+    params[deploy_utils.IPA_URL_PARAM_NAME] = override_callback_url
+
+    inspection_callback = params.pop(inspect_utils.IPA_CALLBACK_PARAM, None)
+    # FIXME(dtantsur): it's much better to stop providing the inspection
+    # callback for built-in inspection, but such a change will require somewhat
+    # newer IPA and thus won't be backportable.
+    if inspection_callback:
+        if inspection_callback.endswith(inspect_utils.CALLBACK_API_ENDPOINT):
+            inspection_callback = inspect_utils.get_inspection_callback(
+                override_callback_url)
+        params[inspect_utils.IPA_CALLBACK_PARAM] = inspection_callback
+
     return params
 
 
@@ -564,8 +576,10 @@ def prepare_deploy_iso(task, params, mode, d_info):
     iso_str = '%s_iso' % mode
     bootloader_str = 'bootloader'
 
-    kernel_href = _find_param(kernel_str, d_info)
-    ramdisk_href = _find_param(ramdisk_str, d_info)
+    # Get kernel and ramdisk, respecting architecture-specific parameters
+    agent_info = driver_utils.get_agent_kernel_ramdisk(task.node, mode)
+    kernel_href = agent_info.get(kernel_str)
+    ramdisk_href = agent_info.get(ramdisk_str)
     iso_href = _find_param(iso_str, d_info)
 
     if not d_info.get('bootloader'):
@@ -589,14 +603,21 @@ def prepare_deploy_iso(task, params, mode, d_info):
         inject_files[_TLS_CONFIG_TEMPLATE.encode('utf-8')] = \
             'etc/ironic-python-agent.d/ironic-tls.conf'
 
-    network_data = task.driver.network.get_node_network_data(task)
-    if network_data:
-        LOG.debug('Injecting custom network data for node %s',
-                  task.node.uuid)
-        network_data = json.dumps(network_data, indent=2).encode('utf-8')
-        inject_files[network_data] = (
-            'openstack/latest/network_data.json'
-        )
+    # Optionally skip injecting network_data for managed inspection when
+    # inspection on all interfaces is requested.
+    skip_network_data = (
+        task.node.provision_state in (states.INSPECTING, states.INSPECTWAIT)
+        and CONF.inspector.force_dhcp
+    )
+    if not skip_network_data:
+        network_data = task.driver.network.get_node_network_data(task)
+        if network_data:
+            LOG.debug('Injecting custom network data for node %s',
+                      task.node.uuid)
+            network_data = json.dumps(network_data, indent=2).encode('utf-8')
+            inject_files[network_data] = (
+                'openstack/latest/network_data.json'
+            )
 
     return prepare_iso_image(inject_files=inject_files)
 
