@@ -10,6 +10,6 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from ironic.drivers.modules.inspector.interface import Inspector
+from ironic.drivers.modules.inspector.agent import AgentInspect
 
-__all__ = ['Inspector']
+__all__ = ['AgentInspect']
