@@ -46,8 +46,7 @@ def _vif_attached(port_like_obj, vif_id):
         False otherwise.
     :raises: VifAlreadyAttached, if vif_id is attached to port_like_obj.
     """
-    attached_vif_id = port_like_obj.internal_info.get(
-        TENANT_VIF_KEY, port_like_obj.extra.get('vif_port_id'))
+    attached_vif_id = port_like_obj.internal_info.get(TENANT_VIF_KEY)
     if attached_vif_id == vif_id:
         raise exception.VifAlreadyAttached(
             object_type=port_like_obj.__class__.__name__,
@@ -233,9 +232,7 @@ def plug_port_to_tenant_network(task, port_like_obj, client=None):
     local_group_info = None
     client_id_opt = None
 
-    vif_id = (
-        port_like_obj.internal_info.get(TENANT_VIF_KEY)
-        or port_like_obj.extra.get('vif_port_id'))
+    vif_id = port_like_obj.internal_info.get(TENANT_VIF_KEY)
 
     if not vif_id:
         obj_name = port_like_obj.__class__.__name__.lower()
@@ -268,11 +265,18 @@ def plug_port_to_tenant_network(task, port_like_obj, client=None):
     # because other port attributes may have been set by the user or
     # nova.
     port_attrs = {'binding:vnic_type': neutron.VNIC_BAREMETAL,
-                  'binding:host_id': node.uuid,
-                  'mac_address': port_like_obj.address}
+                  'binding:host_id': node.uuid}
+    # NOTE(kaifeng) Only update mac address when it's available
+    if port_like_obj.address:
+        port_attrs['mac_address'] = port_like_obj.address
     binding_profile = {'local_link_information': local_link_info}
     if local_group_info:
         binding_profile['local_group_information'] = local_group_info
+
+    # Include physical_network if available
+    if port_like_obj.physical_network:
+        binding_profile['physical_network'] = port_like_obj.physical_network
+
     port_attrs['binding:profile'] = binding_profile
 
     if client_id_opt:
@@ -295,8 +299,27 @@ def plug_port_to_tenant_network(task, port_like_obj, client=None):
 
     try:
         neutron.update_neutron_port(task.context, vif_id, port_attrs)
-        if is_smart_nic:
-            neutron.wait_for_port_status(client, vif_id, 'ACTIVE')
+
+        binding_fail_fatal = False
+        is_neutron_iface = node.network_interface == 'neutron'
+
+        if is_neutron_iface:
+            binding_fail_fatal = getattr(
+                CONF.neutron, 'fail_on_port_binding_failure', False)
+
+        default_failure_behavior = is_smart_nic or binding_fail_fatal
+
+        fail_on_binding_failure = node.driver_info.get(
+            'fail_on_binding_failure', default_failure_behavior)
+
+        # NOTE(cid): Only check port status if it's a smart NIC or if we're
+        # configured to fail on binding failures. This avoids unnecessary
+        # failures when using network interfaces where binding may fail but
+        # we want to continue anyway (like in the case of flat networks).
+        if fail_on_binding_failure:
+            neutron.wait_for_port_status(
+                client, vif_id, 'ACTIVE',
+                fail_on_binding_failure=fail_on_binding_failure)
     except openstack_exc.OpenStackCloudException as e:
         msg = (_('Could not add public network VIF %(vif)s '
                  'to node %(node)s, possible network issue. %(exc)s') %
@@ -305,6 +328,65 @@ def plug_port_to_tenant_network(task, port_like_obj, client=None):
         raise exception.NetworkError(msg)
 
 
+def update_port_host_id(task, vif_id, client=None):
+    """Send an initial host_id value to Neutron to enable allocation.
+
+    :param task: A TaskManager instance.
+    :param vif_id: The VIF ID to "attach" to the host.
+    :param client: A neutron client object
+    :raises: NetworkError if failed to update Neutron port.
+    :raises: VifNotAttached if tenant VIF is not associated with port_like_obj.
+    """
+
+    node = task.node
+
+    if not vif_id:
+        # Nothing to do here, there *is* no VIF to update.
+        return
+
+    if not client:
+        client = neutron.get_client(task.context)
+
+    LOG.debug('Seeding network configuration for VIF %(vif_id)s to node '
+              '%(node_id)s',
+              {'vif_id': vif_id, 'node_id': node.uuid})
+    # We cannot perform an initial/early bind if the port is already
+    # bound. For example, Metalsmith users pre-pass the host_id to
+    # neutron, but then causes this patch to fail in such a case.
+    neutron.unbind_neutron_port_if_bound(vif_id, context=task.context,
+                                         client=client)
+
+    # NOTE(TheJulia): Just provide enough data through to neutron
+    # to start initial binding, but ultimately this entire binding
+    # sequence won't succeed because it cannot succeed. No binding
+    # profile is included, so an ML2 plugin cannot act on the change
+    # and effectively neutron should treat the port update as a NOOP,
+    # except if deferred addressing is the case, then that should
+    # trigger.
+    port_attrs = {'binding:vnic_type': neutron.VNIC_BAREMETAL,
+                  'binding:host_id': node.uuid,
+                  'binding:profile': {}}
+    try:
+        # This cannot be done with the prior/existing client, because they
+        # bring a different context and thus API rights. In other words,
+        # we, as an adminy service need to do this.
+        neutron.update_neutron_port(task.context, vif_id, port_attrs,
+                                    client=None)
+    except openstack_exc.OpenStackCloudException as e:
+        # If his action has failed, the possibility exists that the port
+        # was previously bound *and* allocated to another network, and thus
+        # cannot be properly bound later *either*. This exception should
+        # cause the deployment flow to never be triggered if there is an
+        # underlying issue which only neutron can see like wrong segment
+        # for the vif.
+        msg = (_('Could not seed network configuration for VIF %(vif)s'
+                 'to node %(node)s, possible network, state, or access '
+                 'permission issue. %(exc)s') %
+               {'vif': vif_id, 'node': node.uuid, 'exc': e})
+        LOG.error(msg)
+        raise exception.NetworkError(msg)
+
+
 class VIFPortIDMixin(object):
     """VIF port ID mixin class for non-neutron network interfaces.
 
@@ -363,10 +445,7 @@ class VIFPortIDMixin(object):
         :param port_like_obj: A port or portgroup to check.
         :returns: The ID of the attached VIF, or None.
         """
-        # FIXME(sambetts) Remove this when we no longer support a nova
-        # driver that uses port.extra
-        return (port_like_obj.internal_info.get(TENANT_VIF_KEY)
-                or port_like_obj.extra.get('vif_port_id'))
+        return port_like_obj.internal_info.get(TENANT_VIF_KEY)
 
     def vif_list(self, task):
         """List attached VIF IDs for a node
@@ -402,30 +481,6 @@ class VIFPortIDMixin(object):
                 or p_obj.internal_info.get('inspection_vif_port_id')
                 or self._get_vif_id_by_port_like_obj(p_obj) or None)
 
-    def get_node_network_data(self, task):
-        """Get network configuration data for node's ports/portgroups.
-
-        Gather L2 and L3 network settings from ironic node `network_data`
-        field. Ironic would eventually pass network configuration to the node
-        being managed out-of-band.
-
-        :param task: A TaskManager instance.
-        :raises: InvalidParameterValue, if the network interface configuration
-            is invalid.
-        :raises: MissingParameterValue, if some parameters are missing.
-        :returns: a dict holding network configuration information adhearing
-            Nova network metadata layout (`network_data.json`).
-        """
-        node = task.node
-
-        network_data = node.network_data
-
-        # TODO(etingof): remove or truncate `network_data` logging
-        LOG.debug('Collected network data for node %(node)s: %(data)s',
-                  {'node': node.uuid, 'data': network_data})
-
-        return network_data
-
 
 class NeutronVIFPortIDMixin(VIFPortIDMixin):
     """VIF port ID mixin class for neutron network interfaces.
@@ -604,6 +659,10 @@ class NeutronVIFPortIDMixin(VIFPortIDMixin):
         # NOTE(vsaienko) allow to attach VIF to active instance.
         if task.node.provision_state == states.ACTIVE:
             plug_port_to_tenant_network(task, port_like_obj, client=client)
+        else:
+            # Sends *just* a host_id to trigger neutron to do the initial
+            # address work (if needed...)
+            update_port_host_id(task, vif_id, client=client)
 
     def vif_detach(self, task, vif_id):
         """Detach a virtual network interface from a node
@@ -622,20 +681,23 @@ class NeutronVIFPortIDMixin(VIFPortIDMixin):
         # NOTE(vsaienko): allow to unplug VIFs from ACTIVE instance.
         # NOTE(TheJulia): Also ensure that we delete the vif when in
         # DELETING state.
-        if task.node.provision_state in [states.ACTIVE, states.DELETING]:
+        if task.node.provision_state in [states.ACTIVE, states.DELETING,
+                                         states.AVAILABLE]:
             neutron.unbind_neutron_port(vif_id, context=task.context)
 
     def get_node_network_data(self, task):
         """Get network configuration data for node ports.
 
         Pull network data from ironic node object if present, otherwise
-        collect it for Neutron VIFs.
+        collect it for Neutron VIFs. The intended use of this method is to
+        generate data *for* virtual media based deployments so a minimal
+        configuration can be injected into the ramdisk.
 
         :param task: A TaskManager instance.
         :raises: InvalidParameterValue, if the network interface configuration
             is invalid.
         :raises: MissingParameterValue, if some parameters are missing.
-        :returns: a dict holding network configuration information adhearing
+        :returns: a dict holding network configuration information adhering
             Nova network metadata layout (`network_data.json`).
         """
         # NOTE(etingof): static network data takes precedence
@@ -652,6 +714,10 @@ class NeutronVIFPortIDMixin(VIFPortIDMixin):
         network_data = collections.defaultdict(list)
 
         for port_obj in task.ports:
+            # FIXME(TheJulia) This code ignores portgroups, but also nobody
+            # has complained about this issue *and* it uses get_current_vif
+            # which doesn't handle *tenant* vifs. The model of this is
+            # around deployment, not instances, really.
             vif_port_id = self.get_current_vif(task, port_obj)
 
             LOG.debug('Considering node %(node)s port %(port)s, VIF %(vif)s',
