@@ -11,15 +11,16 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import time
 from urllib.parse import urlparse
 
-from ironic_lib import metrics_utils
 from oslo_log import log
 from oslo_utils import timeutils
 import sushy
 
 from ironic.common import exception
 from ironic.common.i18n import _
+from ironic.common import metrics_utils
 from ironic.common import states
 from ironic.conductor import periodics
 from ironic.conductor import utils as manager_utils
@@ -109,6 +110,27 @@ class RedfishFirmware(base.FirmwareInterface):
             LOG.warning('No manager available to retrieve Firmware '
                         'from the bmc of node %s', task.node.uuid)
 
+        nic_components = None
+        try:
+            nic_components = self.retrieve_nic_components(task, system)
+        except (exception.RedfishError,
+                sushy.exceptions.BadRequestError) as e:
+            # NOTE(janders) if an exception is raised, log a warning
+            # with exception details. This is important for HP hardware
+            # which at the time of writing this are known to return 400
+            # responses to GET NetworkAdapters while OS isn't fully booted
+            LOG.warning('Unable to access NetworkAdapters on node '
+                        '%(node_uuid)s, Error: %(error)s',
+                        {'node_uuid': task.node.uuid, 'error': e})
+        # NOTE(janders) if no exception is raised but no NICs are returned,
+        # state that clearly but in a lower severity message
+        if nic_components == []:
+            LOG.debug('Could not retrieve Firmware Package Version from '
+                      'NetworkAdapters on node %(node_uuid)s',
+                      {'node_uuid': task.node.uuid})
+        elif nic_components:
+            settings.extend(nic_components)
+
         if not settings:
             error_msg = (_('Cannot retrieve firmware for node %s: no '
                            'supported components') % task.node.uuid)
@@ -139,6 +161,33 @@ class RedfishFirmware(base.FirmwareInterface):
                 up_fw_cmp.current_version = up_fw.get('current_version')
                 up_fw_cmp.save()
 
+    def retrieve_nic_components(self, task, system):
+        """Helper function to retrieve all NICs components on a given node.
+
+        :param task: a TaskManager instance.
+        :param system: a Redfish System object
+        :returns: a list of NIC components
+        """
+        nic_list = []
+        try:
+            chassis = redfish_utils.get_chassis(task.node, system)
+        except exception.RedfishError:
+            LOG.warning('No chassis available to retrieve NetworkAdapters '
+                        'firmware information on node %(node_uuid)s',
+                        {'node_uuid': task.node.uuid})
+            return nic_list
+
+        for net_adp in chassis.network_adapters.get_members():
+            for net_adp_ctrl in net_adp.controllers:
+                fw_pkg_v = net_adp_ctrl.firmware_package_version
+                if not fw_pkg_v:
+                    continue
+                net_adp_fw = {'component': redfish_utils.NIC_COMPONENT_PREFIX
+                              + net_adp.identity, 'current_version': fw_pkg_v}
+                nic_list.append(net_adp_fw)
+
+        return nic_list
+
     @METRICS.timer('RedfishFirmware.update')
     @base.deploy_step(priority=0, argsinfo=_FW_SETTINGS_ARGSINFO)
     @base.clean_step(priority=0, abortable=False,
@@ -146,8 +195,7 @@ class RedfishFirmware(base.FirmwareInterface):
                      requires_ramdisk=True)
     @base.service_step(priority=0, abortable=False,
                        argsinfo=_FW_SETTINGS_ARGSINFO,
-                       requires_ramdisk=True)
-    @base.cache_firmware_components
+                       requires_ramdisk=False)
     def update(self, task, settings):
         """Update the Firmware on the node using the settings for components.
 
@@ -174,6 +222,24 @@ class RedfishFirmware(base.FirmwareInterface):
 
         fw_upd = settings[0]
         wait_interval = fw_upd.get('wait')
+
+        # Check if BMC firmware is being updated - BMC may need extended
+        # timeout after firmware update as it transitions through states
+        has_bmc_update = any(
+            setting.get('component', '') == redfish_utils.BMC
+            for setting in settings
+        )
+
+        # Use extended timeout for BMC updates to handle transitional states
+        if has_bmc_update and wait_interval is None:
+            reboot_timeout = CONF.redfish.firmware_update_bmc_timeout
+            LOG.info('BMC firmware update detected, using extended reboot '
+                     'timeout of %(timeout)s seconds for node %(node)s to '
+                     'handle BMC transitional states',
+                     {'timeout': reboot_timeout, 'node': node.uuid})
+        else:
+            reboot_timeout = wait_interval
+
         deploy_utils.set_async_step_flags(
             node,
             reboot=True,
@@ -181,7 +247,8 @@ class RedfishFirmware(base.FirmwareInterface):
             polling=True
         )
 
-        return deploy_utils.reboot_to_finish_step(task, timeout=wait_interval)
+        return deploy_utils.reboot_to_finish_step(task, timeout=reboot_timeout,
+                                                  disable_ramdisk=True)
 
     def _execute_firmware_update(self, node, update_service, settings):
         """Executes the next firmware update to the node
@@ -232,6 +299,28 @@ class RedfishFirmware(base.FirmwareInterface):
                       {'node': node.uuid, 'error': e.message})
             raise exception.RedfishError(error=e)
 
+        # NOTE(iurygregory): In case we are doing firmware updates we need to
+        # account for unresponsive BMC, in this case we wait for a set of
+        # minutes before proceeding to the power actions.
+        # In case the node has firmware_update_unresponsive_bmc_wait set we
+        # give priority over the configuration option.
+        wait_unres_bmc = (
+            node.driver_info.get('firmware_update_unresponsive_bmc_wait')
+            or CONF.redfish.firmware_update_wait_unresponsive_bmc
+        )
+        LOG.debug('BMC firmware update in progress. Waiting %(wait_time)s '
+                  'seconds before proceeding to reboot the node %(node_uuid)s '
+                  'to complete the step', {'node_uuid': node.uuid,
+                                           'wait_time': wait_unres_bmc})
+        # Store task monitor URI for periodic task polling
+        # TODO(iurygregory): Improve the logic here to identify if the BMC
+        # is back, so we don't have to unconditionally wait.
+        # The wait_unres_bmc will be the maximum time to wait.
+        time.sleep(wait_unres_bmc)
+        LOG.debug('Wait completed. Proceeding to reboot the node '
+                  '%(node_uuid)s to complete the step.',
+                  {'node_uuid': node.uuid})
+
         fw_upd['task_monitor'] = task_monitor.task_monitor_uri
         node.set_driver_internal_info('redfish_fw_updates', settings)
 
@@ -243,10 +332,103 @@ class RedfishFirmware(base.FirmwareInterface):
                 fw_clean.append(cleanup)
             node.set_driver_internal_info('firmware_cleanup', fw_clean)
 
+    def _validate_resources_stability(self, node):
+        """Validate that BMC resources are consistently available.
+
+        Requires consecutive successful responses from System, Manager,
+        and NetworkAdapters resources before considering them stable.
+        The number of required successes is configured via
+        CONF.redfish.firmware_update_required_successes.
+        Timeout is configured via
+        CONF.redfish.firmware_update_resource_validation_timeout.
+
+        :param node: the Ironic node object
+        :raises: RedfishError if resources don't stabilize within timeout
+        """
+        timeout = CONF.redfish.firmware_update_resource_validation_timeout
+        required_successes = CONF.redfish.firmware_update_required_successes
+        validation_interval = CONF.redfish.firmware_update_validation_interval
+
+        # Skip validation if validation is disabled via configuration
+        if required_successes == 0 or timeout == 0:
+            reasons = []
+            if required_successes == 0:
+                reasons.append('required_successes=0')
+            if timeout == 0:
+                reasons.append('validation_timeout=0')
+
+            LOG.info('BMC resource validation disabled (%s) for node %(node)s',
+                     ', '.join(reasons), {'node': node.uuid})
+            return
+
+        LOG.debug('Starting resource stability validation for node %(node)s '
+                  '(timeout: %(timeout)s seconds, '
+                  'required_successes: %(required)s, '
+                  'validation_interval: %(interval)s seconds)',
+                  {'node': node.uuid, 'timeout': timeout,
+                   'required': required_successes,
+                   'interval': validation_interval})
+
+        start_time = time.time()
+        end_time = start_time + timeout
+        consecutive_successes = 0
+        last_exc = None
+
+        while time.time() < end_time:
+            try:
+                # Test System resource
+                system = redfish_utils.get_system(node)
+
+                # Test Manager resource
+                redfish_utils.get_manager(node, system)
+
+                # Test NetworkAdapters resource
+                chassis = redfish_utils.get_chassis(node, system)
+                chassis.network_adapters.get_members()
+
+                # All resources successful
+                consecutive_successes += 1
+                LOG.debug('Resource validation success %(count)d/%(required)d '
+                          'for node %(node)s',
+                          {'count': consecutive_successes,
+                           'required': required_successes,
+                           'node': node.uuid})
+
+                if consecutive_successes >= required_successes:
+                    LOG.info('All tested Redfish resources stable and '
+                             ' available for node %(node)s',
+                             {'node': node.uuid})
+                    return
+
+            except (exception.RedfishError,
+                    exception.RedfishConnectionError,
+                    sushy.exceptions.BadRequestError) as e:
+                # Resource not available yet, reset counter
+                if consecutive_successes > 0:
+                    LOG.debug('Resource validation interrupted for node '
+                              '%(node)s, resetting success counter '
+                              '(error: %(error)s)',
+                              {'node': node.uuid, 'error': e})
+                consecutive_successes = 0
+                last_exc = e
+
+            # Wait before next validation attempt
+            time.sleep(validation_interval)
+        # Timeout reached without achieving stability
+        error_msg = _('BMC resources failed to stabilize within '
+                      '%(timeout)s seconds for node %(node)s') % {
+            'timeout': timeout, 'node': node.uuid}
+        if last_exc:
+            error_msg += _(', last error: %(error)s') % {'error': last_exc}
+        LOG.error(error_msg)
+        raise exception.RedfishError(error=error_msg)
+
     def _continue_updates(self, task, update_service, settings):
         """Continues processing the firmware updates
 
         Continues to process the firmware updates on the node.
+        First monitors the current task completion, then validates resource
+        stability before proceeding to next update or completion.
 
         Note that the caller must have an exclusive lock on the node.
 
@@ -256,6 +438,7 @@ class RedfishFirmware(base.FirmwareInterface):
         """
         node = task.node
         fw_upd = settings[0]
+
         wait_interval = fw_upd.get('wait')
         if wait_interval:
             time_now = str(timeutils.utcnow().isoformat())
@@ -394,6 +577,14 @@ class RedfishFirmware(base.FirmwareInterface):
         try:
             task_monitor = redfish_utils.get_task_monitor(
                 node, current_update['task_monitor'])
+        except exception.RedfishConnectionError as e:
+            # If the BMC firmware is being updated, the BMC will be
+            # unavailable for some amount of time.
+            LOG.warning('Unable to communicate with task monitor service '
+                        'on node %(node)s. Will try again on the next poll. '
+                        'Error: %(error)s',
+                        {'node': node.uuid, 'error': e})
+            return
         except exception.RedfishError:
             # The BMC deleted the Task before we could query it
             LOG.warning('Firmware update completed for node %(node)s, '
@@ -422,12 +613,15 @@ class RedfishFirmware(base.FirmwareInterface):
             if (sushy_task.task_state == sushy.TASK_STATE_COMPLETED
                     and sushy_task.task_status in
                     [sushy.HEALTH_OK, sushy.HEALTH_WARNING]):
-                LOG.info('Firmware update succeeded for node %(node)s, '
-                         'firmware %(firmware_image)s: %(messages)s',
+                LOG.info('Firmware update task completed for node %(node)s, '
+                         'firmware %(firmware_image)s: %(messages)s. '
+                         'Starting BMC response validation.',
                          {'node': node.uuid,
                           'firmware_image': current_update['url'],
                           'messages': ", ".join(messages)})
 
+                # Validate BMC resources are consistently available
+                self._validate_resources_stability(node)
                 self._continue_updates(task, update_service, settings)
             else:
                 error_msg = (_('Firmware update failed for node %(node)s, '
