@@ -30,6 +30,7 @@ from ironic.common import exception
 from ironic.common.i18n import _
 from ironic.common import utils
 from ironic.conf import CONF
+from ironic.drivers import utils as driver_utils
 
 LOG = log.getLogger(__name__)
 
@@ -71,7 +72,12 @@ OPTIONAL_PROPERTIES = {
     'redfish_auth_type': _('Redfish HTTP client authentication method. Can be '
                            '"basic", "session" or "auto". If not set, the '
                            'default value is taken from Ironic '
-                           'configuration as ``[redfish]auth_type`` option.')
+                           'configuration as ``[redfish]auth_type`` option.'),
+    'firmware_update_unresponsive_bmc_wait': _(
+        'Number of seconds to wait to avoid the BMC becoming unresponsive '
+        'during firmware updates. If not set, the default value is taken from '
+        'the Ironic configuration ``firmware_update_wait_unresponsive_bmc`` '
+        'in the ``[redfish]`` section.')
 }
 
 COMMON_PROPERTIES = REQUIRED_PROPERTIES.copy()
@@ -85,6 +91,10 @@ BIOS = 'bios'
 BMC = 'bmc'
 "BMC Firmware Component"
 
+NIC_COMPONENT_PREFIX = "nic:"
+"Prefix for NIC Firmware Components"
+
+
 FIRMWARE_COMPONENTS = [BIOS, BMC]
 """Firmware Components available to update"""
 
@@ -151,7 +161,8 @@ def parse_driver_info(node):
                 {'value': driver_info['redfish_system_id'], 'node': node.uuid})
 
     # Check if verify_ca is a Boolean or a file/directory in the file-system
-    verify_ca = driver_info.get('redfish_verify_ca', True)
+    verify_ca = driver_utils.get_verify_ca(
+        node, driver_info.get('redfish_verify_ca', True))
     if isinstance(verify_ca, str):
         if os.path.isdir(verify_ca) or os.path.isfile(verify_ca):
             pass
@@ -193,6 +204,10 @@ def parse_driver_info(node):
     if root_prefix:
         sushy_params['root_prefix'] = root_prefix
 
+    unres_bmc_wait = driver_info.get('firmware_update_unresponsive_bmc_wait')
+    if unres_bmc_wait:
+        sushy_params['firmware_update_unresponsive_bmc_wait'] = unres_bmc_wait
+
     return sushy_params
 
 
@@ -447,7 +462,7 @@ def _get_connection(node, lambda_fun, *args):
         except sushy.exceptions.AccessError as e:
             LOG.warning('For node %(node)s, we received an authentication '
                         'access error from address %(address)s with auth_type '
-                        '%(auth_type)s. The client will not be re-used upon '
+                        '%(auth_type)s. The client will not be reused upon '
                         'the next re-attempt. Please ensure your using the '
                         'correct credentials. Error: %(error)s',
                         {'address': driver_info['address'],
@@ -563,3 +578,26 @@ def get_manager(node, system, manager_id=None):
                   {'system': system.identity,
                    'node': node.uuid, 'error': exc})
         raise exception.RedfishError(error=exc)
+
+
+def get_chassis(node, system):
+    """Get a Redfish Chassis associated with a System of a node
+
+    :param node: an Ironic node object
+    :param system: the Sushy System object
+    :raises: RedfishConnectionError when it fails to connect to Redfish
+    :raises: RedfishError if there is no Chassis registered
+    """
+    try:
+        available_chassis = system.chassis
+        if available_chassis:
+            return available_chassis[0]
+        else:
+            raise exception.RedfishError(
+                _('The Redfish Chassis was not found for node %(node)s.')
+                % {'node': node.uuid})
+    except sushy.exceptions.MissingAttributeError as e:
+        LOG.error('No chassis associated with system %(system)s. '
+                  'Error %(error)s',
+                  {'system': system.identity, 'error': e})
+        raise exception.RedfishError(error=e)
