@@ -219,6 +219,38 @@ class RedfishImageUtilsTestCase(db_base.DbTestCase):
 
             self.assertEqual(expected_url, url)
 
+    @mock.patch.object(image_utils.ImageHandler, 'publish_image',
+                       autospec=True)
+    @mock.patch.object(images, 'create_vfat_image', autospec=True)
+    def test_prepare_floppy_image_with_external_ip_and_inspection(
+            self, mock_create_vfat_image, mock_publish_image):
+        self.config(external_callback_url='http://callback/', group='deploy')
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=True) as task:
+            expected_url = 'https://a.b/c.f?e=f'
+
+            mock_publish_image.return_value = expected_url
+
+            params = {
+                'ipa-api-url': 'http://orig',
+                'ipa-inspection-callback-url':
+                'http://orig/v1/continue_inspection'
+            }
+            url = image_utils.prepare_floppy_image(task, params)
+
+            object_name = 'image-%s.img' % task.node.uuid
+
+            mock_publish_image.assert_called_once_with(mock.ANY, mock.ANY,
+                                                       object_name, None)
+
+            mock_create_vfat_image.assert_called_once_with(
+                mock.ANY, fs_size_kib=1440,
+                parameters={"ipa-api-url": "http://callback",
+                            "ipa-inspection-callback-url":
+                            "http://callback/v1/continue_inspection"})
+
+            self.assertEqual(expected_url, url)
+
     @mock.patch.object(image_utils.ImageHandler, 'publish_image',
                        autospec=True)
     @mock.patch.object(images, 'create_vfat_image', autospec=True)
@@ -871,6 +903,31 @@ class RedfishImageUtilsTestCase(db_base.DbTestCase):
                 task, 'kernel', 'ramdisk', 'bootx64.efi', params={},
                 inject_files={}, base_iso=None)
 
+    @mock.patch.object(image_utils, '_prepare_iso_image', autospec=True)
+    def test_prepare_deploy_iso_kernel_ramdisk_by_arch(
+            self, mock__prepare_iso_image):
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=True) as task:
+
+            cfg.CONF.set_override('deploy_kernel_by_arch',
+                                  {'x86_64': 'file:///images/x86_64-kernel',
+                                   'aarch64': 'file:///images/aarch64-kernel'},
+                                  group='conductor')
+            cfg.CONF.set_override('deploy_ramdisk_by_arch',
+                                  {'x86_64': 'file:///images/x86_64-rd',
+                                   'aarch64': 'file:///images/aarch64-rd'},
+                                  group='conductor')
+
+            task.node.properties['cpu_arch'] = 'aarch64'
+            d_info = {}
+
+            image_utils.prepare_deploy_iso(task, {}, 'deploy', d_info)
+
+            mock__prepare_iso_image.assert_called_once_with(
+                task, 'file:///images/aarch64-kernel',
+                'file:///images/aarch64-rd', None, params={},
+                inject_files={}, base_iso=None)
+
     @mock.patch.object(image_utils, '_prepare_iso_image', autospec=True)
     def test_prepare_deploy_iso_existing_iso(self, mock__prepare_iso_image):
         with task_manager.acquire(self.context, self.node.uuid,
@@ -940,6 +997,35 @@ class RedfishImageUtilsTestCase(db_base.DbTestCase):
                 task, 'kernel', 'ramdisk', bootloader_href=None,
                 params={}, inject_files=expected_files, base_iso=None)
 
+    @mock.patch.object(image_utils, '_prepare_iso_image', autospec=True)
+    def test_prepare_deploy_iso_network_data_skipped_for_inspection(
+            self, mock__prepare_iso_image):
+        # When in inspection and force_dhcp is enabled, ensure
+        # network_data.json is not injected by looking at inject_files to make
+        # sure it is empty.
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=True) as task:
+            cfg.CONF.set_override('force_dhcp', True,
+                                  group='inspector')
+            task.node.provision_state = states.INSPECTING
+
+            d_info = {
+                'deploy_kernel': 'kernel',
+                'deploy_ramdisk': 'ramdisk'
+            }
+            task.node.driver_info.update(d_info)
+
+            # Return some network data, which should be ignored
+            network_data = {'a': ['b']}
+            mock_get_node_nw_data = mock.MagicMock(return_value=network_data)
+            task.driver.network.get_node_network_data = mock_get_node_nw_data
+
+            image_utils.prepare_deploy_iso(task, {}, 'deploy', d_info)
+
+            mock__prepare_iso_image.assert_called_once_with(
+                task, 'kernel', 'ramdisk', bootloader_href=None,
+                params={}, inject_files={}, base_iso=None)
+
     @mock.patch.object(image_utils, '_prepare_iso_image', autospec=True)
     def test_prepare_deploy_iso_tls(self, mock__prepare_iso_image):
         with tempfile.NamedTemporaryFile(delete=False) as tf:
@@ -995,6 +1081,37 @@ cafile = /etc/ironic-python-agent/ironic.crt
                 params={'ipa-api-url': 'http://callback'},
                 inject_files={}, base_iso=None)
 
+    @mock.patch.object(image_utils, '_prepare_iso_image', autospec=True)
+    def test_prepare_deploy_iso_external_ip_with_inspection(
+            self, mock__prepare_iso_image):
+        self.config(external_callback_url='http://callback/', group='deploy')
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=True) as task:
+
+            d_info = {
+                'deploy_kernel': 'kernel',
+                'deploy_ramdisk': 'ramdisk',
+                'bootloader': 'bootloader'
+            }
+            task.node.driver_info.update(d_info)
+
+            task.node.instance_info.update(deploy_boot_mode='uefi')
+
+            params = {
+                'ipa-api-url': 'http://orig',
+                'ipa-inspection-callback-url':
+                'http://orig/v1/continue_inspection'
+            }
+
+            image_utils.prepare_deploy_iso(task, params, 'deploy', d_info)
+
+            mock__prepare_iso_image.assert_called_once_with(
+                task, 'kernel', 'ramdisk', 'bootloader',
+                params={'ipa-api-url': 'http://callback',
+                        'ipa-inspection-callback-url':
+                        'http://callback/v1/continue_inspection'},
+                inject_files={}, base_iso=None)
+
     @mock.patch.object(image_utils, '_find_param', autospec=True)
     @mock.patch.object(image_utils, '_prepare_iso_image', autospec=True)
     @mock.patch.object(images, 'create_boot_iso', autospec=True)
